"use strict";

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj["default"] = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _libVirtualTypes = require("./lib/virtual-types");

var virtualTypes = _interopRequireWildcard(_libVirtualTypes);

var _index = require("../index");

var _index2 = _interopRequireDefault(_index);

var _lodashObjectAssign = require("lodash/object/assign");

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _scope = require("../scope");

var _scope2 = _interopRequireDefault(_scope);

var _types = require("../../types");

var t = _interopRequireWildcard(_types);

var NodePath = (function () {
  function NodePath(parent, container, containerKey) {
    _classCallCheck(this, NodePath);

    this.containerKey = containerKey;
    this.container = container;
    this.contexts = [];
    this.parent = parent;
    this.data = {};
  }

  /**
   * Description
   */

  NodePath.get = function get(_ref) {
    var parentPath = _ref.parentPath;
    var parent = _ref.parent;
    var container = _ref.container;
    var containerKey = _ref.containerKey;
    var key = _ref.key;

    var targetNode = container[key];
    var paths = container._paths = container._paths || [];
    var path;

    for (var i = 0; i < paths.length; i++) {
      var pathCheck = paths[i];
      if (pathCheck.node === targetNode) {
        path = pathCheck;
        break;
      }
    }

    if (!path) {
      path = new NodePath(parent, container, containerKey);
      paths.push(path);
    }

    path.setup(parentPath, key);

    return path;
  };

  /**
   * Description
   */

  NodePath.getScope = function getScope(path, scope, file) {
    var ourScope = scope;

    // we're entering a new scope so let's construct it!
    if (path.isScope()) {
      ourScope = new _scope2["default"](path, scope, file);
    }

    return ourScope;
  };

  /**
   * Description
   */

  NodePath.prototype.setData = function setData(key, val) {
    return this.data[key] = val;
  };

  /**
   * Description
   */

  NodePath.prototype.getData = function getData(key, def) {
    var val = this.data[key];
    if (!val && def) val = this.data[key] = def;
    return val;
  };

  /**
   * Description
   */

  NodePath.prototype.errorWithNode = function errorWithNode(msg) {
    var Error = arguments[1] === undefined ? SyntaxError : arguments[1];

    var loc = this.node.loc.start;
    var err = new Error("Line " + loc.line + ": " + msg);
    err.loc = loc;
    return err;
  };

  /**
   * Description
   */

  NodePath.prototype.traverse = function traverse(visitor, state) {
    (0, _index2["default"])(this.node, visitor, this.scope, state, this);
  };

  return NodePath;
})();

exports["default"] = NodePath;

(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./ancestry"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./resolution"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./replacement"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./evaluation"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./conversion"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./verification"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./context"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./removal"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./modification"));
(0, _lodashObjectAssign2["default"])(NodePath.prototype, require("./family"));

var _loop = function (type) {
  if (type[0] === "_") return "continue";

  NodePath.prototype["is" + type] = function (opts) {
    return virtualTypes[type].checkPath(this, opts);
  };
};

for (var type in virtualTypes) {
  var _ret = _loop(type);

  if (_ret === "continue") continue;
}

var _arr = t.TYPES;

var _loop2 = function () {
  var type = _arr[_i];
  var typeKey = "is" + type;
  NodePath.prototype[typeKey] = function (opts) {
    return t[typeKey](this.node, opts);
  };
};

for (var _i = 0; _i < _arr.length; _i++) {
  _loop2();
}
module.exports = exports["default"];