"use strict";

exports.__esModule = true;
exports.getTypeAnnotation = getTypeAnnotation;
exports.resolve = resolve;
exports.inferType = inferType;
exports.isTypeGeneric = isTypeGeneric;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj["default"] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _lodashLangIsBoolean = require("lodash/lang/isBoolean");

var _lodashLangIsBoolean2 = _interopRequireDefault(_lodashLangIsBoolean);

var _lodashLangIsNumber = require("lodash/lang/isNumber");

var _lodashLangIsNumber2 = _interopRequireDefault(_lodashLangIsNumber);

var _lodashLangIsString = require("lodash/lang/isString");

var _lodashLangIsString2 = _interopRequireDefault(_lodashLangIsString);

var _types = require("../../types");

var t = _interopRequireWildcard(_types);

/**
 * Description
 */

function getTypeAnnotation() {
  if (this.typeInfo) {
    return this.typeInfo;
  }

  var info = this.typeInfo = {
    inferred: false,
    annotation: null
  };

  var type = this.node && this.node.typeAnnotation;

  if (!type) {
    info.inferred = true;
    type = this.inferType(this);
  }

  if (type) {
    if (t.isTypeAnnotation(type)) type = type.typeAnnotation;
    info.annotation = type;
  }

  return info;
}

/**
 * Description
 */

function resolve(resolved) {
  // detect infinite recursion
  if (resolved && resolved.indexOf(this) >= 0) return;

  // we store all the paths we've "resolved" in this array to prevent infinite recursion
  resolved = resolved || [];
  resolved.push(this);

  if (this.isVariableDeclarator()) {
    if (this.get("id").isIdentifier()) {
      return this.get("init").resolve(resolved);
    } else {}
  } else if (this.isIdentifier()) {
    var binding = this.scope.getBinding(this.node.name);
    if (!binding || !binding.constant) return;

    if (binding.path === this) {
      return this;
    } else {
      return binding.path.resolve(resolved);
    }
  } else if (this.isMemberExpression()) {
    // this is dangerous, as non-direct target assignments will mutate it's state
    // making this resolution inaccurate

    var targetKey = this.toComputedKey();
    if (!t.isLiteral(targetKey)) return;
    var targetName = targetKey.value;

    var target = this.get("object").resolve(resolved);
    if (!target || !target.isObjectExpression()) return;

    var props = target.get("properties");
    for (var i = 0; i < props.length; i++) {
      var prop = props[i];
      if (!prop.isProperty()) continue;

      var key = prop.get("key");

      // { foo: obj }
      var match = prop.isnt("computed") && key.isIdentifier({ name: targetName });

      // { "foo": "obj" } or { ["foo"]: "obj" }
      match = match || key.isLiteral({ value: targetName });

      if (match) return prop.get("value");
    }
  } else {
    return this;
  }
}

/**
 * Description
 */

function inferType(path) {
  path = path.resolve();
  if (!path) return;

  if (path.isPreviousType("RestElement") || path.parentPath.isPreviousType("RestElement") || path.isPreviousType("ArrayExpression")) {
    return t.genericTypeAnnotation(t.identifier("Array"));
  }

  if (path.parentPath.isPreviousType("TypeCastExpression")) {
    return path.parentPath.node.typeAnnotation;
  }

  if (path.isPreviousType("TypeCastExpression")) {
    return path.node.typeAnnotation;
  }

  if (path.isPreviousType("ObjectExpression")) {
    return t.genericTypeAnnotation(t.identifier("Object"));
  }

  if (path.isPreviousType("Function")) {
    return t.identifier("Function");
  }

  if (path.isPreviousType("Literal")) {
    var value = path.node.value;
    if ((0, _lodashLangIsString2["default"])(value)) return t.stringTypeAnnotation();
    if ((0, _lodashLangIsNumber2["default"])(value)) return t.numberTypeAnnotation();
    if ((0, _lodashLangIsBoolean2["default"])(value)) return t.booleanTypeAnnotation();
  }

  if (path.isPreviousType("CallExpression")) {
    var callee = path.get("callee").resolve();
    if (callee && callee.isPreviousType("Function")) return callee.node.returnType;
  }
}

/**
 * Description
 */

function isTypeGeneric(genericName) {
  var opts = arguments[1] === undefined ? {} : arguments[1];

  var typeInfo = this.getTypeAnnotation();
  var type = typeInfo.annotation;
  if (!type) return false;

  if (typeInfo.inferred && opts.inference === false) {
    return false;
  }

  if (!t.isGenericTypeAnnotation(type) || !t.isIdentifier(type.id, { name: genericName })) {
    return false;
  }

  if (opts.requireTypeParameters && !type.typeParameters) {
    return false;
  }

  return true;
}

// otherwise it's a request for a pattern and that's a bit more tricky